#    Copyright 2025 Genesis Corporation.
#
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


class Rule:
    def __init__(self, service, res, perm):
        self.service = service
        self.res = res
        self.perm = perm

    def __str__(self):
        return f"{self.service}.{self.res}.{self.perm}"

    @classmethod
    def from_raw(cls, rule):
        service, res, perm = rule.split(".", maxsplit=2)

        return cls(service, res, perm)
